package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.AuthStatusDTO;
import com.diagbot.dto.QueryAuthProgressDTO;
import com.diagbot.dto.UserAuthenticationDTO;
import com.diagbot.dto.WaitAuthenDTO;
import com.diagbot.entity.Organization;
import com.diagbot.entity.User;
import com.diagbot.entity.UserAuthentication;
import com.diagbot.enums.AuthStatusEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.UserAuthenticationServiceImpl;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.UserAuthenticationVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * @Description:用户认证业务层
 * @author: zhaops
 * @time: 2018/9/13 19:30
 */
@Component
public class UserAuthenticationFacade extends UserAuthenticationServiceImpl {
    @Autowired
    private UserFacade userFacade;
    @Autowired
    private OrganizationFacade organizationFacade;

    /**
     * 用户认证
     *
     * @param userAuthenticationVO
     * @return
     */
    public AuthStatusDTO userAuthentication(UserAuthenticationVO userAuthenticationVO) {
        Long userId = Long.parseLong(UserUtils.getCurrentPrincipleID());
        User user_old = userFacade.getById(userId);
        User user = userFacade.findByName(userAuthenticationVO.getUsername());
        if (user == null) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS,
                    "用户【" + userAuthenticationVO.getUsername() + "】不存在，不允许验证");
        } else if (!userId.equals(user.getId())) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "登录用户【" + user_old.getUsername() + "】与验证用户【" + user.getUsername() + "】不符，不允许验证");
        }

        //判断该用户是否有关联机构
        Organization organization_old = organizationFacade.getByUserId(userId);
        if (organization_old == null) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS,
                    "当前用户没有关联机构，不允许验证");
        }
        //更新机构信息
        QueryWrapper<Organization> qwOrganization = new QueryWrapper<>();
        qwOrganization.eq("name", userAuthenticationVO.getOrganization()).
                eq("is_deleted", IsDeleteEnum.N.getKey());
        Organization organization = organizationFacade.getOne(qwOrganization);
        if (organization == null) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS,
                    "找不到机构【" + userAuthenticationVO.getOrganization() + "】");
        } else if (!organization.getId().equals(organization_old.getId())) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "当前用户已关联到机构【" + organization_old.getName() + "】,不允许修改机构");
        }
        organization.setAddress(userAuthenticationVO.getOrganizationAddress());
        organization.setPrincipal(userAuthenticationVO.getOrganizationPrincipal());
        organization.setType(userAuthenticationVO.getOrganizationType());
        organization.setModifier(userId.toString());
        organization.setGmtModified(new Date());
        organization.setSubNum(userAuthenticationVO.getSubOrganizationNum());
        organizationFacade.updateById(organization);

        //更新验证信息
        QueryWrapper<UserAuthentication> qwAuthentication = new QueryWrapper<>();
        qwAuthentication.eq("user_id", userId).
                eq("is_deleted", IsDeleteEnum.N.getKey());
        UserAuthentication userAuthentication = this.getOne(qwAuthentication);
        if (userAuthentication == null) {
            userAuthentication = new UserAuthentication();
            userAuthentication.setCreator(userId.toString());
            userAuthentication.setGmtCreate(new Date());
            userAuthentication.setUserId(userId);
        } else if (userAuthentication.getStatus().equals(AuthStatusEnum.Authorized.getKey())) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "用户【" + user.getUsername() + "】已认证，不允许重复认证");
        } else if (userAuthentication.getStatus().equals(AuthStatusEnum.Authorizing.getKey())) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "认证申请已提交，无需重复提交");
        }
        userAuthentication.setModifier(userId.toString());
        userAuthentication.setGmtModified(new Date());
        userAuthentication.setPosition(userAuthenticationVO.getPosition());
        userAuthentication.setStatus(AuthStatusEnum.Authorizing.getKey());  //状态设为认证中
        this.saveOrUpdate(userAuthentication);

        AuthStatusDTO authStatusDTO = new AuthStatusDTO();
        authStatusDTO.setAuthStatus(userAuthentication.getStatus());
        authStatusDTO.setAuthStatusName(AuthStatusEnum.Authorizing.getName());
        return authStatusDTO;
    }

    /**
     * 获取待认证用户信息
     *
     * @return
     */
    public UserAuthenticationDTO getuserAuthenticationInfo() {
        Long userId = Long.parseLong(UserUtils.getCurrentPrincipleID());
        User user = userFacade.getById(userId);
        //密码置空
        user.setPassword(null);
        Organization organization = organizationFacade.getByUserId(userId);
        QueryWrapper<UserAuthentication> qw = new QueryWrapper<>();
        qw.eq("user_id", userId).
                eq("is_deleted", IsDeleteEnum.N.getKey());
        UserAuthentication userAuthentication = this.getOne(qw);
        UserAuthenticationDTO userAuthenticationDTO = new UserAuthenticationDTO();
        //userAuthenticationDTO.setUser(user);
        userAuthenticationDTO.setUserId(user.getId());
        userAuthenticationDTO.setUsername(user.getUsername());
        userAuthenticationDTO.setLinkman(user.getLinkman());
        userAuthenticationDTO.setEmail(user.getEmail());
        userAuthenticationDTO.setUserType(user.getType());
        //userAuthenticationDTO.setOrganization(organization);
        userAuthenticationDTO.setOrganizationId(organization.getId());
        userAuthenticationDTO.setOrganizationName(organization.getName());
        userAuthenticationDTO.setOrganizationType(organization.getType());
        userAuthenticationDTO.setPrincipal(organization.getPrincipal());
        userAuthenticationDTO.setSubNum(organization.getSubNum());
        userAuthenticationDTO.setAddress(organization.getAddress());
        //userAuthenticationDTO.setUserAuthentication(userAuthentication);
        userAuthenticationDTO.setPosition(userAuthentication.getPosition());
        userAuthenticationDTO.setIsReject(userAuthentication.getIsReject());
        userAuthenticationDTO.setRejectType(userAuthentication.getRejectType());
        userAuthenticationDTO.setRejectComment(userAuthentication.getRejectComment());
        userAuthenticationDTO.setAuthStatus(userAuthentication.getStatus());
        userAuthenticationDTO.setAuthStatusName(AuthStatusEnum.getName(userAuthentication.getStatus()));
        return userAuthenticationDTO;
    }

    /**
     * 获取用户认证状态
     *
     * @return
     */
    public AuthStatusDTO getUserAuthenticationStatus() {
        UserAuthentication userAuthentication = this.getUserAuthentication();
        AuthStatusDTO authStatusDTO = new AuthStatusDTO();
        Integer status = userAuthentication.getStatus();
        String statusName = AuthStatusEnum.getName(status);
        authStatusDTO.setAuthStatus(status);
        authStatusDTO.setAuthStatusName(statusName);
        return authStatusDTO;
    }

    /**
     * 控制台-获取用户账号的认证进度
     *
     * @return 包含当前账号的认证状态
     */
    public QueryAuthProgressDTO queryAuthProgress() {
        UserAuthentication userAuthentication = this.getUserAuthentication();

        QueryAuthProgressDTO queryAuthProgressDTO = new QueryAuthProgressDTO();
        queryAuthProgressDTO.setUserStatus(userAuthentication.getStatus());
        queryAuthProgressDTO.setUserStatusMsg(AuthStatusEnum.getName(userAuthentication.getStatus()));
        queryAuthProgressDTO.setRejectComment(userAuthentication.getRejectComment());

        return queryAuthProgressDTO;
    }

    /**
     * 获取用户认证信息
     *
     * @return 用户认证信息
     */
    private UserAuthentication getUserAuthentication() {
        Long userId = Long.parseLong(UserUtils.getCurrentPrincipleID());
        QueryWrapper<UserAuthentication> qw = new QueryWrapper<>();
        qw.eq("user_id", userId).
                eq("is_deleted", IsDeleteEnum.N.getKey());
        UserAuthentication userAuthentication = this.getOne(qw);
        if (userAuthentication == null) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "当前用户没有认证信息");
        }
        return userAuthentication;
    }

    /**
     * 控制台-获取待认证的账号数量
     *
     * @return 待认证的账号个数
     */
    public WaitAuthenDTO waitAuthen() {
        WaitAuthenDTO waitAuthenDTO = new WaitAuthenDTO();
        waitAuthenDTO.setCount(baseMapper.getAllWaitAuthCou());
        return waitAuthenDTO;
    }
}
